/****************************************************************************************
**     FRETsg v1.0 (C) 2002   Gunnar F. Schrder,  Helmut Grubmller    (Gttingen)    **
*****************************************************************************************/


#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <math.h>
#include <malloc.h>

#define TRUE (1)
#define FALSE (0)
#define PI 3.14159265358979323846
#define DIM 3
#define XX 0
#define YY 1
#define ZZ 2
#define MAX_COO 999.9
#define MAX_STRING_SIZE 9999
#define MAX_NR_POSITIONS 399
#define MAX_NR_DISTANCES 1999
#define MAX_NR_CONFIGURATIONS 2000
#define GRADIENT_THRESHOLD 0.005
#define FORCE_CONSTANT 5.0
#define NR_DIFFUSION_STEPS 500
#define NR_MC_STEPS 10
#define MAX_NR_DIFFUSION_STEPS 10000
#define RANDOM_FORCE 1.2
#define RANDOM_FORCE_MINIMIZATION 0.8
#define MIN_STEP 0.02
#define DIFFUSION_STEP 0.03
#define MAX_MIN_STEP 2.0
#define TFACTOR 2.0

typedef double          real;
typedef real            rvec[DIM];
typedef real            matrix[DIM][DIM];


#define ROTATE(a,i,j,k,l) g=a[i][j];h=a[k][l];a[i][j]=g-s*(h+g*tau);\
  a[k][l]=h+s*(g-h*tau);


/**************** functions ***************************************************/

static  void     terminate(void);
static  void     read_positions(void);
static  void     read_distances(void);
static  void     init_coordinates(void);	
static  void     minimize(void);
static  void     run_diffusion(void);
static  void     create_configurations(void);
static  double   ran2(long *idum);
static  double   rang(void);
static  void     init_grad(void);
static  void     save_configurations(void);
static  void     jacobi(double **a,int n,double d[],double **v,int *nrot);
static  double   do_fit(int natoms,rvec *xp,rvec *x);
static  void     calc_fit_R(int natoms,rvec *xp,rvec *x,matrix R);
static  void     reset_x(int natoms,rvec x[]);
static  void     clear_mat(matrix a);
static  void     oprod(rvec a,rvec b,rvec c);
static  void     read_input_file(void);
int              read_int_skip(FILE *f);
double           read_double_skip (FILE * f);
void             skip_line (FILE * f,int n);
int              read_string_skip(FILE *f,char *str);
int              read_bool_skip(FILE *f);
void             add_mirror_to_name(char *str);
void             check_pdb_output_filename(char *str);
char             get_char(FILE *f);
static  void     calc_distances_of_known(void);
static  void     make_povname(char *str);
static  void     write_pov_input(void);
static  void     fluct_ana(void);









/**************** variables ***************************************************/

static char 	positionsfile[MAX_STRING_SIZE],
		distancesfile[MAX_STRING_SIZE],
		outputfile[MAX_STRING_SIZE],
                mirror_outputfile[MAX_STRING_SIZE],
		input_filename[MAX_STRING_SIZE],
                pov_filename[MAX_STRING_SIZE];
static int      position_labels[MAX_NR_POSITIONS];
static int      label_fit_flag[MAX_NR_POSITIONS];
static int      fixed_labels[MAX_NR_POSITIONS];
static float 	position_x[MAX_NR_POSITIONS],
		position_y[MAX_NR_POSITIONS],
		position_z[MAX_NR_POSITIONS];
static int      nr_positions,nr_distances;
static int      distance_list_label1[MAX_NR_DISTANCES],
	        distance_list_label2[MAX_NR_DISTANCES];
static float    distance_list[MAX_NR_DISTANCES];
static float    distance_width[MAX_NR_DISTANCES];
static double   atom_x[MAX_NR_POSITIONS],
	        atom_y[MAX_NR_POSITIONS],
	        atom_z[MAX_NR_POSITIONS];
static double   xcoo[MAX_NR_POSITIONS][MAX_NR_CONFIGURATIONS+1],
	        ycoo[MAX_NR_POSITIONS][MAX_NR_CONFIGURATIONS+1],
	        zcoo[MAX_NR_POSITIONS][MAX_NR_CONFIGURATIONS+1];
static double   temp_factor[MAX_NR_POSITIONS][MAX_NR_CONFIGURATIONS+1];
static double   t_tempfactor[MAX_NR_POSITIONS];

static double	grad_x[MAX_NR_POSITIONS],
		grad_y[MAX_NR_POSITIONS],
		grad_z[MAX_NR_POSITIONS];
static int 	natoms_fit;
static int 	*fit_labels;
static int 	nr_configurations,diffusion_mode,
		nr_diffusion_steps,write_mirror_flag,
                pov_flag;
       char  	*oo="..ooOOoo"; 
       char   	*ooo="...oooOOOooo"; 
static float    pov_colors[MAX_NR_POSITIONS][3];
static double   minimization_threshold;

static  long int  seed;

#define M 714025
#define IA 1366
#define IC 150889

static double ran2(long *idum)
{
        static long iy,ir[98];
        static int iff=0;
        int j;

        if (*idum < 0 || iff == 0) {
                iff=1;
                if ((*idum=(IC-(*idum)) % M) < 0) *idum = -(*idum);
                for (j=1;j<=97;j++) {
                        *idum=(IA*(*idum)+IC) % M;
                        ir[j]=(*idum);
                }
                *idum=(IA*(*idum)+IC) % M;
                iy=(*idum);
        }
        j=1 + 97.0*iy/M;
        if (j > 97 || j < 1) {
          fprintf(stderr,"\n\nERROR: This should really not happen. Sorry.\n");
          exit(1);
          }
        iy=ir[j];
        *idum=(IA*(*idum)+IC) % M;
        ir[j]=(*idum);
        return (double) iy/M;
}

#undef M
#undef IA
#undef IC

static double rang(void)
/* Gibt Zufallsvariable mit in 8.Ordnung angenaeherter Gaussverteilung
   um 0.0 und Varianz 1.0 */
{
  int i;
  double r;

  r=0.0;
  for (i=1; i<=8; i++) {
    r=r+ran2(&seed);
    }
/*
        Zentrieren:
        v=v-8*1/2
        Mitteln:
        v=v/8
        1/(2*sqrt(3)) ist Varianz der Gleichverteilung in [0,1]:
        v=v*2*sqrt(3)
        Varianz der Summe von n Zufallszahlen geht mit 1/sqrt(n):
        v=v*sqrt(8)
        daher:
        v=(v-4)/sqrt(3/2)
*/
  return (r-4.0)*1.2247448713;
}


void   add_mirror_to_name(char *str)
{

   char    str1[MAX_STRING_SIZE];
   char    *str3=".pdb";
   int     length,suffix_length;
   char     *str2="_mirror";
   char     *suffix_pos;


   length=strlen(str);
   suffix_pos=strstr(str,str3);
   suffix_length=strlen(suffix_pos);
   
   if (suffix_length!=4) {
      strcpy(str1,str);
      strcat(str1,str2);
      strcat(str1,str3); 

   } else {

      strncpy(str1,str,length-4);
      strcat(str1,str2);
      strcat(str1,str3);
   }      

  strcpy(str,str1);
}


char  get_char(FILE *f)
{
   char c;
   fpos_t position;

   fgetpos(f,&position);
   c=fgetc(f);
   fsetpos(f,&position);

   return c;
}


int     read_int_skip(FILE *f)
{
        int    i;
        int    ch;

        fscanf (f, "%d", &i);
        while (((ch = fgetc (f)) != EOF) && (ch != ((int)'\n'))) {}
        return i;
}


double  read_double_skip (FILE *f)
{
        double x;
        int    ch;

#if PRECISION == DOUBLE
        fscanf(f,"%le",&x);
#else
        fscanf(f,"%e",&x);
#endif
        while (((ch=fgetc (f)) != EOF) && (ch != ((int)'\n'))) {}
        return x;
}


int     read_bool_skip(FILE *f)
{
        char   str[MAX_STRING_SIZE];
        int    i;
        int    ch;

        fscanf(f,"%s",str);
        if (strcmp(str,"TRUE")==0) {
                i = TRUE;
        } else
        if (strcmp(str,"FALSE")==0) {
                i = FALSE;
        } else {
                fprintf(stderr,"Fatal error: Found no BOOL-Var in this line:\n");
                fprintf(stderr,"%s\n",str);
                i = -1;
        }
        while (((ch = fgetc (f)) != EOF) && (ch != ((int)'\n'))) {}
        return i;
}


void    skip_line (FILE *f,int n)
{
        int    i;
        int    ch;

        for (i = 0; i < n; i++) {
                while (((ch = fgetc (f)) != EOF) && (ch != ((int)'\n'))) {}
        } /* for i */
}


int     read_string_skip(FILE *f,char *str)
{
        int     i,ch;
        i=0;
        str[0]=0;
        do {
                ch=fgetc(f);
                if ((ch==EOF)||(ch=='\n')||(ch==' ')||(ch=='\t')) {
                        str[i]=0;
                        if (ch==EOF)    return -1;
                        if ((ch==' ')||(ch=='\t'))    {
      		             while (((ch = fgetc (f)) != EOF) && (ch != ((int)'\n'))) {}	
			     return 0;
              		}
                } else {
                        str[i++]= (char)ch;
                }
                if (i>=MAX_STRING_SIZE-2) {
                   fprintf(stderr,"Fatal error: Line too long for string-buffer (read_string).\n");
                   fprintf(stderr,"Actual stringbuffer-size is %d.\n",MAX_STRING_SIZE);
                   str[i]=0;
                   return -1;
                }
        } while ((ch!=' ') && (ch!=EOF) && (ch!='\t'));
        str[i]=0;
        return -1;
}




static void clear_mat(matrix a)
{
  const real nul=0.0;

  a[XX][XX]=a[XX][YY]=a[XX][ZZ]=nul;
  a[YY][XX]=a[YY][YY]=a[YY][ZZ]=nul;
  a[ZZ][XX]=a[ZZ][YY]=a[ZZ][ZZ]=nul;
}


static void oprod(rvec a,rvec b,rvec c)
{
  c[XX]=a[YY]*b[ZZ]-a[ZZ]*b[YY];
  c[YY]=a[ZZ]*b[XX]-a[XX]*b[ZZ];
  c[ZZ]=a[XX]*b[YY]-a[YY]*b[XX];
}


static void terminate()
{
  fprintf(stderr,"\n\n terminated with fatal error.\n\n");
  exit(1);
}


static void make_povname(char *str)
{
    char *str1=".inp";
    char *pov_str=".pov";
    char *substr;
    char  str2[MAX_STRING_SIZE];
    size_t   length;

    length=strlen(str);

    if (( substr=strstr(str,str1)) != NULL ) {
       if (strlen(substr) == 4) {
             strncpy(str2,str,length-4);  
             str2[length-4]=0;
             strcat(str2,pov_str);
             strcpy(str,str2);
       } else strcat(str,pov_str); 
    } else strcat(str,pov_str);
          

}


void check_pdb_output_filename(char *str)
{
   char    str1[MAX_STRING_SIZE];
   char    *str2=".pdb";
   int     length,suffix_length;
   char     *suffix_pos;


   strcpy(str1,str);
   length=strlen(str1);

   if ((suffix_pos=strstr(str1,str2)) !=NULL ) {
       suffix_length=strlen(suffix_pos);
       if (suffix_length!=4) {
            strcat(str1,str2);
            strcpy(str,str1);

       } else strcpy(str,str1);

   } else {

        strcat(str1,str2);
        strcpy(str,str1);
   }

}

static void read_input_file()
{

   FILE *fp;
   char ch;

   if ((fp=fopen(input_filename,"r")) == NULL) {
	fprintf(stderr,"\n\nERROR: Could not read input file %s\n\n",input_filename);
	exit(1);
   }

  fprintf(stderr,"  Reading input parameters from file  %s ...\n",input_filename);

  /***** Skip comment lines  ******/
  while ((ch=get_char(fp)) == '#') skip_line(fp,1);


  read_string_skip(fp,positionsfile);
  read_string_skip(fp,distancesfile);
  read_string_skip(fp,outputfile);
  check_pdb_output_filename(outputfile);

  nr_configurations=read_int_skip(fp);
  if (nr_configurations > MAX_NR_CONFIGURATIONS) {
	fprintf(stderr,"\n\nERROR:  number of configurations must smaller than %i\n\n",
		                                      MAX_NR_CONFIGURATIONS);
        exit(1);
  }
  if (nr_configurations <= 0) {
	fprintf(stderr,"\n\nERROR:  number of configurations must be larger than 1\n\n");
        exit(1);
  }

  minimization_threshold=read_double_skip(fp);
  if ((minimization_threshold <= 0.0) || (minimization_threshold > 1.0)) {
        fprintf(stderr,"\n\nERROR:  Gradient Threshold must be larger than 0.0 and smaller than 1.0\n\n");
        exit(1);
  }


  diffusion_mode=read_bool_skip(fp);


  nr_diffusion_steps=read_int_skip(fp);
  if ((diffusion_mode) && (nr_diffusion_steps < 1)) {
	fprintf(stderr,"\n\nERROR:  number of diffusion steps must be larger than 0.\n\n");
        exit(1);
  }

  if ((diffusion_mode) && (nr_diffusion_steps > MAX_NR_DIFFUSION_STEPS)) {
	fprintf(stderr,"\n\nERROR:  number of diffusion steps must be smaller than %i.\n\n",
				MAX_NR_DIFFUSION_STEPS);
        exit(1);
  }

  write_mirror_flag=read_bool_skip(fp);
  pov_flag=read_bool_skip(fp);

  fclose(fp);
  
/*
  fprintf(stderr,"Positionsfilename:  %s\n",positionsfile);
  fprintf(stderr,"Distancesfilename:  %s\n",distancesfile);
  fprintf(stderr,"Outputfilename:  %s\n",outputfile);
  fprintf(stderr,"nr_configurations: %i\n",nr_configurations);
  fprintf(stderr,"diffusion_mode: %i \n",diffusion_mode);
  fprintf(stderr,"nr_diffusion_steps: %i\n",nr_diffusion_steps);
  fprintf(stderr,"write_mirror: %i\n",write_mirror_flag);
*/



}

static void read_positions()
{
      FILE *fp;
      int i,j,h;
      char ch;
 
      if ((fp=fopen(positionsfile,"r"))==NULL) {
          fprintf(stderr,"\n\nERROR: Could not read from file %s\n\n",positionsfile);
          terminate();
      }
      fprintf(stderr,"  Reading position labels from file   %s ...\n",positionsfile);

      /***** Skip comment lines  ******/
      while ((ch=get_char(fp)) == '#') skip_line(fp,1);

      nr_positions=0; 
      do 
      {
        fscanf(fp,"%i",&position_labels[nr_positions]);
        if (position_labels[nr_positions] < 0) {
		fprintf(stderr,"\n\nERROR:  Position labels must be positive integer.\n\n");
		exit(1);
	}

        fscanf(fp,"%i",&fixed_labels[nr_positions]);
        if ((fixed_labels[nr_positions] !=0) && (fixed_labels[nr_positions] !=1)) {
           fprintf(stderr,"\n\nERROR:  Switch to indicate known atom positions must be 0 or 1\n\n");
           exit(1);
        }

        fscanf(fp,"%i",&label_fit_flag[nr_positions]);
        if ((label_fit_flag[nr_positions] !=0) && (label_fit_flag[nr_positions] !=1)) {
           fprintf(stderr,"\n\nERROR:  Switch to indicate labels for fitting must be 0 or 1\n\n");
           exit(1);
        }


    if (fixed_labels[nr_positions]==1) {

        fscanf(fp,"%g",&position_x[nr_positions]);
        if (abs(position_x[nr_positions]) > MAX_COO) {
           fprintf(stderr,"\n\nERROR:  x-coordinate of label %i is too large.\n\n",
				position_labels[nr_positions]);
           exit(1);
        }

        fscanf(fp,"%g",&position_y[nr_positions]);
        if (abs(position_y[nr_positions]) > MAX_COO) {
           fprintf(stderr,"\n\nERROR:  y-coordinate of label %i is too large.\n\n",
				position_labels[nr_positions]);
           exit(1);
        }

        fscanf(fp,"%g",&position_z[nr_positions]);
        if (abs(position_z[nr_positions]) > MAX_COO) {
           fprintf(stderr,"\n\nERROR:  z-coordinate of label %i is too large.\n\n",
				position_labels[nr_positions]);
           exit(1);
        }

     } else {
       skip_line(fp,1);
     }


  	nr_positions++;
        if (nr_positions > MAX_NR_POSITIONS) { 
		fprintf(stderr,"\n\nERROR:  Too many position labels defined ( max %i )\n\n",
      				MAX_NR_POSITIONS);
		exit(1);
	}


      } while ( position_labels[nr_positions-1] > 0);
	nr_positions--;
      fclose(fp);

   if (nr_positions <= 2) {
       fprintf(stderr,"\n\nERROR:  number of labels must be larger than 2.\n\n");
       exit(1);
   }

   natoms_fit=0;
   for (i=0;i<nr_positions;i++) if (label_fit_flag[i] == 1)	natoms_fit++;
   
   fit_labels=(int *) calloc(natoms_fit,sizeof(int));
   j=0;
   for (i=0;i<nr_positions;i++)  
	if (label_fit_flag[i] == 1) fit_labels[j++]=i;

   h=0; 
   for (i=0;i<nr_positions;i++)  
        if ((fixed_labels[i]==1) && (label_fit_flag[i]==0)) h++;

   if (h!=0) {
	fprintf(stderr,"\n\nWARNING:  There are positions declared \"known\", which are not used for fitting !!\n\n");
   }

}

static void read_distances()
{

      FILE *fp;
      int i,j,h;
      char ch;

      fprintf(stderr,"  Reading distance pairs from file    %s ...\n ",distancesfile);
      if ((fp=fopen(distancesfile,"r"))==NULL) {
          fprintf(stderr,"\n\nERROR: Could not read from file %s\n\n",distancesfile);
          exit(1);
      }
      nr_distances=0;

      /***** Skip comment lines  ******/
      while ((ch=get_char(fp)) == '#') skip_line(fp,1);

      do
      {
	distance_list_label1[nr_distances]=-9999;

        fscanf(fp,"%i",&distance_list_label1[nr_distances]);
        fscanf(fp,"%i",&distance_list_label2[nr_distances]);
        fscanf(fp,"%g",&distance_list[nr_distances]);
        fscanf(fp,"%g",&distance_width[nr_distances]);

        nr_distances++;
      } while ( distance_list_label1[nr_distances-1] != -9999);
	nr_distances--;
      fclose(fp);

   /*** Check consistency of position and distance labels  *****/
   for (j=0;j<nr_distances;j++) {
      h=0;
      for (i=0;i<nr_positions;i++) 
           if (distance_list_label1[j]==position_labels[i]) h++;
      if (h==0) {
                fprintf(stderr,"\n\nERROR:  Label %i not defined in positions file.\n\n",
                        distance_list_label1[j]);
                exit(1);
        }

      h=0;
      for (i=0;i<nr_positions;i++) 
           if (distance_list_label2[j]==position_labels[i]) h++;
      if (h==0) {
                fprintf(stderr,"\n\nERROR:  Label %i not defined in positions file.\n\n",
                        distance_list_label2[j]);
                exit(1);
        }
      
      if (distance_list[j] < 0.001) {
                fprintf(stderr,"\n\nERROR:  Distances must be larger than 0.0 .\n\n");
                exit(1);
        }
   }

}

/*** If the positions of some atoms are known, then the distances are  *******
**** calculated and used in the subsequent model building              *******/
static void calc_distances_of_known()
{
   int       altdist_label1[MAX_NR_DISTANCES],
             altdist_label2[MAX_NR_DISTANCES];
   int       i,j,k,h;
   int       alt_nr_distances;
   double    altdistlist[MAX_NR_DISTANCES];
   double    d,f;

  alt_nr_distances=0;
  for (i=0;i<nr_positions;i++) { 
   for (j=i+1;j<nr_positions;j++) { 
     if ((fixed_labels[i]==1) && (fixed_labels[j]==1)) {
       h=0;
       for (k=0;k<nr_distances;k++) {
         if (((distance_list_label1[k]==position_labels[i]) && 
                                 (distance_list_label2[k]==position_labels[j])) ||
             ((distance_list_label2[k]==position_labels[i]) &&
                                  (distance_list_label1[k]==position_labels[j]))) h++;
       }
   
       if (h==0) {
         altdist_label1[alt_nr_distances]=position_labels[i];
         altdist_label2[alt_nr_distances]=position_labels[j];
         
         f=position_x[i]-position_x[j];   
         d=f*f;
         f=position_y[i]-position_y[j];   
         d+=f*f;
         f=position_z[i]-position_z[j];   
         d+=f*f;
         d=sqrt(d);
         altdistlist[alt_nr_distances]=d;
         alt_nr_distances++;
       }
          
     }
   }
 }

   /*** Add new distances to the ones from the distances file  ***/
   for (i=0;i<alt_nr_distances;i++) {
      distance_list_label1[nr_distances+i]=altdist_label1[i];  
      distance_list_label2[nr_distances+i]=altdist_label2[i];  
      distance_list[nr_distances+i]=altdistlist[i]; 
   }
   nr_distances+=alt_nr_distances;
   

}
		
static void init_coordinates()
{
    int i;

	for (i=0;i<nr_positions;i++) {
/*		atom_x[i]=(ran2(&seed)-0.5)*1000.0;
		atom_y[i]=(ran2(&seed)-0.5)*1000.0;
		atom_z[i]=(ran2(&seed)-0.5)*1000.0; */

/*		atom_x[i]=rang()*500.0;
		atom_y[i]=rang()*500.0;
		atom_z[i]=rang()*500.0; */


		atom_x[i]=rang()*50.0;
		atom_y[i]=rang()*50.0;
		atom_z[i]=rang()*50.0;
	}
}

static void init_grad()
{
    int i;
    for (i=0;i<MAX_NR_POSITIONS;i++) {
	grad_x[i]=0.0;
	grad_y[i]=0.0;
	grad_z[i]=0.0;
    }
}

static void minimize()
{
	int i,i1,i2,j,k;
	int step_count;
	double max_grad,norm_gradient,norm_gradient_1;
	double d,f,d_1,minimize_step;
	double E[MAX_NR_POSITIONS];


	norm_gradient=0.0;
	norm_gradient_1=0.0;
	minimize_step=0.0;
	step_count=0;
           
      do 
      {
	init_grad();
	for (i=0;i<nr_positions;i++) E[i]=0.0;
	max_grad=0.0;
	for (j=0;j<nr_distances;j++) {
		for (i=0;i<nr_positions;i++) { 
			if (position_labels[i]==distance_list_label1[j]) i1=i; }
		for (i=0;i<nr_positions;i++) { 
			if (position_labels[i]==distance_list_label2[j]) i2=i; }

		f=(atom_x[i1]-atom_x[i2]);
		d=(f*f);
		f=(atom_y[i1]-atom_y[i2]);
                d+=(f*f);
		f=(atom_z[i1]-atom_z[i2]);
		d+=(f*f);
		d=sqrt(d);
		d_1=1.0/d;
		grad_x[i1]-=FORCE_CONSTANT*(d-(double )distance_list[j])*(atom_x[i1]-atom_x[i2])*d_1;
		grad_y[i1]-=FORCE_CONSTANT*(d-(double )distance_list[j])*(atom_y[i1]-atom_y[i2])*d_1;
		grad_z[i1]-=FORCE_CONSTANT*(d-(double )distance_list[j])*(atom_z[i1]-atom_z[i2])*d_1;

		grad_x[i2]+=FORCE_CONSTANT*(d-(double )distance_list[j])*(atom_x[i1]-atom_x[i2])*d_1;
		grad_y[i2]+=FORCE_CONSTANT*(d-(double )distance_list[j])*(atom_y[i1]-atom_y[i2])*d_1;
		grad_z[i2]+=FORCE_CONSTANT*(d-(double )distance_list[j])*(atom_z[i1]-atom_z[i2])*d_1;

		f=d-(double )distance_list[j];
		f=f*f;
		E[i1]+=0.5*FORCE_CONSTANT*f;
		E[i2]+=0.5*FORCE_CONSTANT*f;
        }

	for (i=0;i<nr_positions;i++) {
		norm_gradient=0.0;
		f=grad_x[i];  f*=f;
		norm_gradient+=f;
		f=grad_y[i];  f*=f;
		norm_gradient+=f;
		f=grad_z[i];  f*=f;
		norm_gradient+=f;

	norm_gradient=sqrt(norm_gradient);
	norm_gradient_1=1.0/norm_gradient;

		grad_x[i]*=norm_gradient_1;
		grad_y[i]*=norm_gradient_1;
		grad_z[i]*=norm_gradient_1;



	minimize_step=norm_gradient*MIN_STEP;
	if (minimize_step > MAX_MIN_STEP) minimize_step=MAX_MIN_STEP;
	
		step_count++;	
		atom_x[i]+=minimize_step*grad_x[i]+
			(1.0/(double )step_count)*RANDOM_FORCE_MINIMIZATION*rang();
		atom_y[i]+=minimize_step*grad_y[i]+
			(1.0/(double )step_count)*RANDOM_FORCE_MINIMIZATION*rang();
		atom_z[i]+=minimize_step*grad_z[i]+
			(1.0/(double )step_count)*RANDOM_FORCE_MINIMIZATION*rang();

                if (minimize_step > max_grad ) max_grad=norm_gradient;

	}


     } while (max_grad > minimization_threshold);	


        for (i=0;i<nr_positions;i++) {
              t_tempfactor[i]=E[i]*TFACTOR;
/*              printf("E(%i)= %f\n",i,E[i]);*/
        }
	
}


static void run_diffusion()
{

        int i,k,l,j,i1,i2,ll;
        double d,norm_gradient,f;
	double random_force;
	double 	x0[MAX_NR_POSITIONS],
		y0[MAX_NR_POSITIONS],
		z0[MAX_NR_POSITIONS];
	double sum_gradient[MAX_NR_POSITIONS];

   random_force=RANDOM_FORCE;
   ll=0;
   fprintf(stderr,"  Run Fluctuation Analysis   ............... ");

   for (i=0;i<nr_positions;i++) sum_gradient[i]=0.0;
 

   for (l=0;l< nr_configurations;l++) {

       if (++ll > 11 )  ll-=12;                

       /*fprintf(stderr,"\b%c",ooo[ll]);*/
       fprintf(stderr,"\b\b\b\b%3d%%",(int)(floor((l*100.0)/(nr_configurations-1.0))));

       /*fflush(stderr);*/



             for (i=0;i<nr_positions;i++) {
               x0[i]=atom_x[i]=xcoo[i][l];
               y0[i]=atom_y[i]=ycoo[i][l];
               z0[i]=atom_z[i]=zcoo[i][l];
             }

     for (k=0;k<NR_DIFFUSION_STEPS;k++) {

        init_grad();
        for (j=0;j<nr_distances;j++) {

                for (i=0;i<nr_positions;i++) {
                        if (position_labels[i]==distance_list_label1[j]) i1=i; }
                for (i=0;i<nr_positions;i++) {
                        if (position_labels[i]==distance_list_label2[j]) i2=i; }

		f=atom_x[i1]-x0[i2]; 
		d=(f*f);
		f=atom_y[i1]-y0[i2];
		d+=(f*f);
		f=atom_z[i1]-z0[i2];
		d+=(f*f);
                d=sqrt(d);

                grad_x[i1]-=FORCE_CONSTANT*(d-(double )distance_list[j])*(atom_x[i1]-x0[i2])/d;
                grad_y[i1]-=FORCE_CONSTANT*(d-(double )distance_list[j])*(atom_y[i1]-y0[i2])/d;
                grad_z[i1]-=FORCE_CONSTANT*(d-(double )distance_list[j])*(atom_z[i1]-z0[i2])/d;

                f=x0[i1]-atom_x[i2];
                d=(f*f);
                f=y0[i1]-atom_y[i2];
                d+=(f*f);
                f=z0[i1]-atom_z[i2];
                d+=(f*f);
                d=sqrt(d);

                grad_x[i2]+=FORCE_CONSTANT*(d-(double )distance_list[j])*(x0[i1]-atom_x[i2])/d;
                grad_y[i2]+=FORCE_CONSTANT*(d-(double )distance_list[j])*(y0[i1]-atom_y[i2])/d;
                grad_z[i2]+=FORCE_CONSTANT*(d-(double )distance_list[j])*(z0[i1]-atom_z[i2])/d;

         }


        for (i=0;i<nr_positions;i++) {
              norm_gradient=0.0;
              f=grad_x[i]; f*=f;
              norm_gradient+=f;
              f=grad_y[i]; f*=f;
              norm_gradient+=f;
              f=grad_z[i]; f*=f;
              norm_gradient+=f;  norm_gradient=sqrt(norm_gradient);
	      sum_gradient[i]+=norm_gradient;



/*                atom_x[i]+=DIFFUSION_STEP*grad_x[i]+random_force*rang();
                atom_y[i]+=DIFFUSION_STEP*grad_y[i]+random_force*rang();
                atom_z[i]+=DIFFUSION_STEP*grad_z[i]+random_force*rang();
*/

                atom_x[i]+=DIFFUSION_STEP*grad_x[i]+random_force*(ran2(&seed)-0.5);
                atom_y[i]+=DIFFUSION_STEP*grad_y[i]+random_force*(ran2(&seed)-0.5);
                atom_z[i]+=DIFFUSION_STEP*grad_z[i]+random_force*(ran2(&seed)-0.5);
        }


    }

             for (i=0;i<nr_positions;i++) {
               xcoo[i][l]=atom_x[i];
               ycoo[i][l]=atom_y[i];
               zcoo[i][l]=atom_z[i];
             }
   }

/*     for (i=0;i<nr_positions;i++) printf("Gradientensumme %i = %f\n",i,sum_gradient[i]/
          ((double)nr_configurations *(double) NR_DIFFUSION_STEPS));
*/

   fprintf(stderr,"\b\b\b\b.... Done.\n");

}

/***** Monte-Carlo Simulation ************/
static void fluct_ana()
{
        int i,k,l,j,i1,i2,ll;
        double d,dd,norm_gradient,f;
	double E,Eold,mc_step;
        double  x0[MAX_NR_POSITIONS],
                y0[MAX_NR_POSITIONS],
                z0[MAX_NR_POSITIONS];
        int distlist[MAX_NR_POSITIONS][MAX_NR_POSITIONS];
        double dist[MAX_NR_POSITIONS][MAX_NR_POSITIONS];
        double dist_width[MAX_NR_POSITIONS][MAX_NR_POSITIONS];
        double force_const[MAX_NR_POSITIONS][MAX_NR_POSITIONS];
        int ndist[MAX_NR_POSITIONS];
 	double atom_x_t[MAX_NR_POSITIONS];
 	double atom_y_t[MAX_NR_POSITIONS];
 	double atom_z_t[MAX_NR_POSITIONS];
	
   fprintf(stderr,"  Run Fluctuation Analysis   ............... ");


    for (i=0;i<nr_positions;i++) ndist[i]=0;
    for (i=0;i<nr_positions;i++) 
      for (j=0;j<nr_positions;j++) distlist[i][j]=-1;

    for (i=0;i<nr_positions;i++) {
      for (j=0;j<nr_positions;j++) {
        for (l=0;l<nr_distances;l++) {
	  if (((position_labels[i]==distance_list_label1[l]) && 
			(position_labels[j]==distance_list_label2[l])) ||
               ((position_labels[i]==distance_list_label2[l]) &&
                        (position_labels[j]==distance_list_label1[l]))) {
		distlist[i][ndist[i]]=j;
		dist[i][ndist[i]]=(double )distance_list[l];
		dist_width[i][ndist[i]]=(double )distance_width[l];
                force_const[i][ndist[i]]=1.0/((double) distance_width[l]*(double) distance_width[l]);
		ndist[i]++;
 	  }
        }
      }
     } 

/*    for (i=0;i<nr_positions;i++) {
      for (j=0;j<ndist[i];j++) {
	printf("dist_width[%i][%i]= %f \n",i,j,dist_width[i][j]);
      }
    } */

    
   for (l=0;l< nr_configurations;l++) {

       fprintf(stderr,"\b\b\b\b%3d%%",(int)(floor((l*100.0)/(nr_configurations-1.0))));


    for (i=0;i<nr_positions;i++) {
          x0[i]=atom_x[i]=xcoo[i][l];
          y0[i]=atom_y[i]=ycoo[i][l];
          z0[i]=atom_z[i]=zcoo[i][l];
    }


    for (i=0;i<nr_positions;i++) {
       k=0;
       mc_step=9999.0;
       for (j=0;j<ndist[i];j++) {
		if (mc_step > dist_width[i][j]) mc_step=dist_width[i][j];
       }
       mc_step/=2.0;
       Eold=99999.0;
       do {
 
	  atom_x_t[i]=atom_x[i]+mc_step*rang();
	  atom_y_t[i]=atom_y[i]+mc_step*rang();
	  atom_z_t[i]=atom_z[i]+mc_step*rang();

          E=0;
          for (j=0;j<ndist[i];j++) {
		f=atom_x_t[i]-x0[distlist[i][j]];
		f*=f;
		d=f;
		f=atom_y_t[i]-y0[distlist[i][j]];
		f*=f;
		d+=f;
		f=atom_z_t[i]-z0[distlist[i][j]];
		f*=f;
		d+=f;
		d=sqrt(d);
  		dd=d- dist[i][j];
		E+=0.5*force_const[i][j]*dd*dd;
          }		
          
          if ( (E <= Eold) || (ran2(&seed) < exp(Eold-E))) {
                atom_x[i]=atom_x_t[i];
                atom_y[i]=atom_y_t[i];
                atom_z[i]=atom_z_t[i];
		Eold=E;	
		k++;
		  
   	  } 


       } while (k < NR_MC_STEPS) ;

      xcoo[i][l]=atom_x[i];
      ycoo[i][l]=atom_y[i];
      zcoo[i][l]=atom_z[i];
     }
   }

   fprintf(stderr,"\b\b\b\b.... Done.\n");

}


static void create_configurations()
{

	int i,k,l,j,i1,i2,ll;
        double d;

   ll=0; 
   fprintf(stderr,"\n  Creating %4i minimized Configurations ... ",nr_configurations);

   for (l=0;l< nr_configurations;l++) {

/*       if (++ll > 7 )  ll-=8;                */

       fprintf(stderr,"\b\b\b\b%3d%%",(int)(floor((l*100.0)/(nr_configurations-1.0))));

/*       fprintf(stderr,"\b%c",oo[ll]);
       fflush(stderr); */

       init_coordinates(); 
       minimize(); 

  
  
       for (i=0;i<nr_positions;i++) {
            xcoo[i][l]=atom_x[i];
            ycoo[i][l]=atom_y[i];
            zcoo[i][l]=atom_z[i];

            temp_factor[i][l]=t_tempfactor[i];
       }

   }
   fprintf(stderr,"\b\b\b\b.... Done.\n");
}

static void save_configurations()
{
	FILE *fp;
	int i,k;
	char atom_name[MAX_STRING_SIZE+1],
		residue_name[MAX_STRING_SIZE+1],
		chain_id[MAX_STRING_SIZE+1];
	float occupancy;


   if ((fp=fopen(outputfile,"w")) ==NULL) {
          fprintf(stderr,"\n\nERROR: Could not write to file %s\n",outputfile);
          terminate();
      }

   fprintf(stderr,"  Writing Output to  %s  .\n",outputfile);

	strcpy(atom_name,"C");
	strcpy(residue_name,"LYS");
	strcpy(chain_id,"A");
	occupancy=1.0;

 for (k=0;k<nr_configurations;k++) {
   fprintf(fp,"MODEL     %4i\n",k+1);
   for (i=0;i<nr_positions;i++) {

      fprintf(fp,"ATOM  %5i  %-4s%-4s%5i    %8.3f%8.3f%8.3f %5.2f %5.2f   %4s\n",i+1,
		atom_name,residue_name,position_labels[i],xcoo[i][k],ycoo[i][k],zcoo[i][k],
		temp_factor[i][k],temp_factor[i][k],chain_id);
   }
   fprintf(fp,"ENDMDL\n");
 }
 fprintf(fp,"END\n");
 fclose(fp);



 if (write_mirror_flag) {
   
     strcpy(mirror_outputfile,outputfile);
     add_mirror_to_name(mirror_outputfile); 
     if ((fp=fopen(mirror_outputfile,"w")) ==NULL) {
              fprintf(stderr,"\n\nERROR: Could not write to file %s\n",mirror_outputfile);
              terminate();
          }
    
   fprintf(stderr,"  Writing Mirror Output to  %s  .\n",mirror_outputfile);

            strcpy(atom_name,"C");
            strcpy(residue_name,"LYS");
            strcpy(chain_id,"A");
            occupancy=1.0;
    
     for (k=0;k<nr_configurations;k++) {
       fprintf(fp,"MODEL     %4i\n",k+1);
       for (i=0;i<nr_positions;i++) {
    
          fprintf(fp,"ATOM  %5i  %-4s%-4s%5i    %8.3f%8.3f%8.3f %5.2f %5.2f   %4s\n",i+1,
                    atom_name,residue_name,position_labels[i],-xcoo[i][k],ycoo[i][k],zcoo[i][k],
                    occupancy,temp_factor[i][k],chain_id);
       }
       fprintf(fp,"ENDMDL\n");
     }
     fprintf(fp,"END\n");
     fclose(fp);


 }



}

static void set_pov_colors()
{
    int i;

   for (i=0;i<nr_positions;i++) {
       pov_colors[i][0]=pov_colors[i][1]=pov_colors[i][2]=0.5;
   }

   pov_colors[0][0]=1.0;
   pov_colors[0][1]=0.0;
   pov_colors[0][2]=0.0;

   pov_colors[1][0]=0.0;
   pov_colors[1][1]=1.0;
   pov_colors[1][2]=0.0;

   pov_colors[2][0]=0.0;
   pov_colors[2][1]=0.0;
   pov_colors[2][2]=1.0;

   pov_colors[3][0]=0.0;
   pov_colors[3][1]=0.7;
   pov_colors[3][2]=0.7;

   pov_colors[4][0]=0.7;
   pov_colors[4][1]=0.0;
   pov_colors[4][2]=0.7;

   pov_colors[5][0]=0.7;
   pov_colors[5][1]=0.7;
   pov_colors[5][2]=0.0;

   pov_colors[6][0]=0.3;
   pov_colors[6][1]=0.9;
   pov_colors[6][2]=0.2;
        
   

}

static void write_pov_input()
{
    int i,j,k;
    double   z_shift;
    FILE *fp;
    double coc[3];

    if ((fp=fopen(pov_filename,"w"))==NULL)  {
          fprintf(stderr,"\n\nERROR:  Could not write PovRay output to file %s\n",pov_filename);
          exit(1);
    }

    fprintf(stderr,"  Writing POVRay Output to file  %s \n",pov_filename); 
    set_pov_colors();

    coc[0]=coc[1]=coc[2]=0.0;
    for (k=0;k<nr_configurations;k++) { 
       for (i=0;i<nr_positions;i++) coc[0]+=xcoo[i][k];
       for (i=0;i<nr_positions;i++) coc[1]+=ycoo[i][k];
       for (i=0;i<nr_positions;i++) coc[2]+=zcoo[i][k];
    }
    coc[0]/=(double) (nr_configurations*nr_positions);
    coc[1]/=(double) (nr_configurations*nr_positions);
    coc[2]/=(double) (nr_configurations*nr_positions);

    z_shift=70.0;
    fprintf(fp,"//************************************************************\n");
    fprintf(fp,"//**  This POVRay input file was created by CheckFRET v1.0  **\n");
    fprintf(fp,"//************************************************************\n\n\n");
    fprintf(fp,"camera {\n  location <%f,%f,%f>\n",coc[0],coc[1],coc[2]+z_shift);
    fprintf(fp,"  up <0.0,1.0,0.0> \n");
    fprintf(fp,"  right <1.0,0.0,0.0> \n");
    fprintf(fp,"  look_at <%f,%f,%f>\n} \n",coc[0],coc[1],coc[2]);
    fprintf(fp,"light_source { <%f,%f,%f> rgb <1.0,1.0,1.0> }\n",coc[0],coc[1]+5.0,coc[2]+z_shift);
    fprintf(fp,"background { \n  color rgb<1.00, 1.00, 1.00>\n} \n");
    fprintf(fp,"\n#declare sphere_radius=3.0;\n\n");
    fprintf(fp,"\n#declare blob_strength=1.0;\n\n");

    for (i=0;i<nr_positions;i++) {
      fprintf(fp,"blob  {\n  threshold 0.6\n");
      for (j=0;j<nr_configurations;j++) {
         fprintf(fp,"  sphere { <%f,%f,%f>,sphere_radius,blob_strength }\n",xcoo[i][j],ycoo[i][j],zcoo[i][j]);
      }
      fprintf(fp,"  pigment { rgb <%1.2f, %1.2f, %1.2f> }\n",pov_colors[i][0],pov_colors[i][1],pov_colors[i][2]);
      fprintf(fp,"  finish { ambient 0.3  phong 0.62 } \n");
      fprintf(fp,"}\n\n\n"); 
    }

}




/*** This routine is adapted from the GROMACS source code  ******/
static double do_fit(int natoms,rvec *xp,rvec *x)
{
  int    i,ai,j,m,r,c;
  matrix R;
  rvec   x_old;
  rvec   *ind_x,*ind_xp;
  double ah,rms;


  ind_xp=(rvec *) calloc(natoms_fit,sizeof(rvec)); 
  ind_x=(rvec *) calloc(natoms_fit,sizeof(rvec)); 
  for (i=0;i<natoms_fit;i++) {
         ai=fit_labels[i];
         ind_xp[i][XX]=xp[ai][XX];  
         ind_xp[i][YY]=xp[ai][YY];  
         ind_xp[i][ZZ]=xp[ai][ZZ];  
         ind_x[i][XX]=x[ai][XX];  
         ind_x[i][YY]=x[ai][YY];  
         ind_x[i][ZZ]=x[ai][ZZ];  
  }  

 
  /* Calculate the rotation matrix R */
   calc_fit_R(natoms_fit,ind_xp,ind_x,R); 


  /*rotate X*/
  for(j=0; j<natoms; j++) {
    for(m=0; m<DIM; m++)
      x_old[m]=x[j][m];
    for(r=0; r<DIM; r++) {
      x[j][r]=0;
      for(c=0; c<DIM; c++)
        x[j][r]+=R[r][c]*x_old[c];
    }
  }
  /* Calculation of RMS */
  rms=0.0;
  for (i=0;i<natoms_fit;i++) {
    ai=fit_labels[i];
    for (m=0;m<DIM;m++) {
      ah=xp[ai][m]-x[ai][m];
      ah*=ah;
      rms+=ah;
    }
  }


 free(ind_xp);
 free(ind_x); 

 return sqrt(rms);

}


/*** This routine is adapted from the GROMACS source code  ******/
static void calc_fit_R(int natoms,rvec *xp,rvec *x,matrix R)
{

  int    c,r,n,j,m,i,irot;
  static double **omega=NULL,**om=NULL;
  double d[2*DIM],xnr,xpc;
  matrix vh,vk,u;
  real   mn;
  int    index;
  real   max_d;

  if (omega == NULL) {
	omega=(double **) calloc(2*DIM,sizeof(double));
	om=(double **) calloc(2*DIM,sizeof(double));
    for(i=0; i<2*DIM; i++) {
	omega[i]=(double *) calloc(2*DIM,sizeof(double));
	om[i]=(double *) calloc(2*DIM,sizeof(double));
    }
  }

  for(i=0; i<2*DIM; i++) {
    d[i]=0;
    for(j=0; j<2*DIM; j++) {
      omega[i][j]=0;
      om[i][j]=0;
    }
  }

  /*calculate the matrix U*/
  clear_mat(u);
  for(n=0;(n<natoms);n++)
      for(c=0; (c<DIM); c++) {
        xpc=xp[n][c];
        for(r=0; (r<DIM); r++) {
          xnr=x[n][r];
          u[c][r]+=xnr*xpc;
        }
      }

  /*construct omega*/
  /*omega is symmetric -> omega==omega' */
  for(r=0; r<2*DIM; r++)
    for(c=0; c<=r; c++)
      if (r>=DIM && c<DIM) {
        omega[r][c]=u[r-DIM][c];
        omega[c][r]=u[r-DIM][c];
      } else {
        omega[r][c]=0;
        omega[c][r]=0;
      }

  /*determine h and k*/
  jacobi(omega,2*DIM,d,om,&irot);
  /*real   **omega = input matrix a[0..n-1][0..n-1] must be symmetric
   *int     natoms = number of rows and columns
   *real      NULL = d[0]..d[n-1] are the eigenvalues of a[][]
   *real       **v = v[0..n-1][0..n-1] contains the vectors in columns
   *int      *irot = number of jacobi rotations
   */
  /* Copy only the first two eigenvectors */
  for(j=0; j<2; j++) {
    max_d=-1000;
    for(i=0; i<2*DIM; i++)
      if (d[i]>max_d) {
        max_d=d[i];
        index=i;
      }
    d[index]=-10000;
    for(i=0; i<DIM; i++) {
      vh[j][i]=M_SQRT2*om[i][index];
      vk[j][i]=M_SQRT2*om[i+DIM][index];
    }
  }
  /* Calculate the last eigenvector as the outer-product of the first two.
   * This insures that the conformation is not mirrored and
   * prevents problems with completely flat reference structures.
   */
  oprod(vh[0],vh[1],vh[2]);
  oprod(vk[0],vk[1],vk[2]);

  /*determine R*/
  for(r=0; r<DIM; r++)
    for(c=0; c<DIM; c++)
      R[r][c] = vk[0][r]*vh[0][c] +
                vk[1][r]*vh[1][c] +
                vk[2][r]*vh[2][c];

}


/*** This routine is adapted from the GROMACS source code  ******/
static void reset_x(int natoms,rvec x[])
{

   rvec xcm;
   int i,m,ai;
   rvec   *ind_x;


  ind_x=(rvec *) calloc(natoms_fit,sizeof(rvec));
  for (i=0;i<natoms_fit;i++) {
         ai=fit_labels[i];
         ind_x[i][XX]=x[ai][XX];
         ind_x[i][YY]=x[ai][YY];
         ind_x[i][ZZ]=x[ai][ZZ];
  }

   xcm[XX]=0.0; xcm[YY]=0.0; xcm[ZZ]=0.0;
   for (i=0;i<natoms_fit;i++) 
       for(m=0; m<DIM; m++)
 	    xcm[m]+=ind_x[i][m];
             
  free(ind_x);

  for(m=0; m<DIM; m++)
    xcm[m]/=(double )natoms_fit;

  for (i=0;i<natoms;i++) 
      for(m=0; m<DIM; m++)
           x[i][m]-=xcm[m];



}


/* This routine was taken from NUMERICAL RECIPES */
static void jacobi(double **a,int n,double d[],double **v,int *nrot)
{
  int j,i;
  int iq,ip;
  double tresh,theta,tau,t,sm,s,h,g,c,*b,*z;

/*  snew(b,n);
  snew(z,n);  */
  b=(double *) calloc(n,sizeof(double));
  z=(double *) calloc(n,sizeof(double));
  for (ip=0; ip<n; ip++) {
    for (iq=0; iq<n; iq++) v[ip][iq]=0.0;
    v[ip][ip]=1.0;
 }
  for (ip=0; ip<n;ip++) {
    b[ip]=d[ip]=a[ip][ip];
    z[ip]=0.0;
  }
  *nrot=0;
  for (i=1; i<=50; i++) {
    sm=0.0;
    for (ip=0; ip<n-1; ip++) {
      for (iq=ip+1; iq<n; iq++)
        sm += fabs(a[ip][iq]);
    }
    if (sm == 0.0) {
/*      sfree(z);
      sfree(b);  */
   if (z != NULL )  free(z);
   if (b != NULL )  free(b);
      return;
    }
    if (i < 4)
      tresh=0.2*sm/(n*n);
    else
      tresh=0.0;
    for (ip=0; ip<n-1; ip++) {
      for (iq=ip+1; iq<n; iq++) {
        g=100.0*fabs(a[ip][iq]);
        if (i > 4 && fabs(d[ip])+g == fabs(d[ip])
            && fabs(d[iq])+g == fabs(d[iq]))
          a[ip][iq]=0.0;
        else if (fabs(a[ip][iq]) > tresh) {
          h=d[iq]-d[ip];
          if (fabs(h)+g == fabs(h))
            t=(a[ip][iq])/h;
          else {
            theta=0.5*h/(a[ip][iq]);
            t=1.0/(fabs(theta)+sqrt(1.0+theta*theta));
            if (theta < 0.0) t = -t;
          }
          c=1.0/sqrt(1+t*t);
          s=t*c;
          tau=s/(1.0+c);
          h=t*a[ip][iq];
          z[ip] -= h;
          z[iq] += h;
          d[ip] -= h;
          d[iq] += h;
          a[ip][iq]=0.0;
          for (j=0; j<ip; j++) {
            ROTATE(a,j,ip,j,iq)
          }
          for (j=ip+1; j<iq; j++) {
            ROTATE(a,ip,j,j,iq)
            }
          for (j=iq+1; j<n; j++) {
            ROTATE(a,ip,j,iq,j)
            }
          for (j=0; j<n; j++) {
            ROTATE(v,j,ip,j,iq)
            }
          ++(*nrot);
        }
      }
    }
    for (ip=0; ip<n; ip++) {
      b[ip] +=  z[ip];
      d[ip]  =  b[ip];
      z[ip]  =  0.0;
    }
  }
  fprintf(stderr,"Too many iterations in routine JACOBI");
}


void bad_input()
{

 fprintf(stderr,"\n  Usage :  checkfret <InputFileName> \n\n"); 

}

int main(int argc,char *argv[])
{
	int i,j,count;
        rvec *xp,*x,*x_mirror;
	double rms_l,rms_r;
  

        seed=1566; 


        
        fprintf(stderr,"\n\n     FRETsg 1.0  \n");
        fprintf(stderr,"     (C) 2002 Gunnar F. Schrder  and  Helmut Grubmller\n");
        fprintf(stderr,"     3D Model Creation from multiple FRET distances.\n");
        fprintf(stderr,"\n\n");


	if (argc == 1) {
                bad_input();
		fprintf(stderr,"\n\nERROR: You must specify an InputFilename !!!\n");
		exit(1);
  	}
    
        if (argc > 2) {
                bad_input();
                fprintf(stderr,"\nERROR: Too many arguments !!!\n\n");
		exit(1);
	}
         

        strcpy(input_filename,argv[1]);
        read_input_file();
	read_positions();
	read_distances();

	calc_distances_of_known();

	init_coordinates();	
	minimize();
	create_configurations();

	xp=(rvec *) calloc(nr_positions, sizeof(rvec));
	x=(rvec *) calloc(nr_positions, sizeof(rvec));
	x_mirror=(rvec *) calloc(nr_positions, sizeof(rvec));

	for (i=0;i<nr_positions;i++) {
			xp[i][XX]=xcoo[i][0];
			xp[i][YY]=ycoo[i][0];
			xp[i][ZZ]=zcoo[i][0];
	}
        reset_x(nr_positions,xp);
        for (i=0;i<nr_positions;i++) {
             xcoo[i][0]=xp[i][XX];
             ycoo[i][0]=xp[i][YY];
             zcoo[i][0]=xp[i][ZZ];
        }

     count=0;
     for (j=1;j<nr_configurations;j++) {
	for (i=0;i<nr_positions;i++) {
			x[i][XX]=xcoo[i][j];
			x[i][YY]=ycoo[i][j];
			x[i][ZZ]=zcoo[i][j];
	}
				
	reset_x(nr_positions,x);
	rms_l=do_fit(nr_positions,xp,x); 
      
        /* build the mirror structure */
        for (i=0;i<nr_positions;i++) {
                        x_mirror[i][XX]=-xcoo[i][j];
                        x_mirror[i][YY]=ycoo[i][j];
                        x_mirror[i][ZZ]=zcoo[i][j];
        }
	reset_x(nr_positions,x_mirror);
        rms_r=do_fit(nr_positions,xp,x_mirror); 
      if (rms_l <= rms_r) {
        for (i=0;i<nr_positions;i++) {
             xcoo[i][j]=x[i][XX];
             ycoo[i][j]=x[i][YY];
             zcoo[i][j]=x[i][ZZ];
        }
      } else {
        count++;
        for (i=0;i<nr_positions;i++) {
             xcoo[i][j]=x_mirror[i][XX];
             ycoo[i][j]=x_mirror[i][YY];
             zcoo[i][j]=x_mirror[i][ZZ];
        }
      }

     }
/*    if (diffusion_mode==1) run_diffusion();*/
    if (diffusion_mode==1) fluct_ana();
 
    save_configurations();
    
    if (pov_flag==1) { 
       strcpy(pov_filename,input_filename);
       make_povname(pov_filename);
       write_pov_input();
    }

    fprintf(stderr,"\n");

    free(x);
    free(x_mirror);
    free(xp);

}
